/*         ______   ___    ___
 *        /\  _  \ /\_ \  /\_ \
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      Allegro bitmap -> Windows icon converter.
 *
 *      By Charles Wardlaw.
 * 
 *      See readme.txt for copyright information.      
 */
 
#import <Cocoa/Cocoa.h>
#import <Carbon/Carbon.h>

#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include <allegro.h>
#import "IconFamily.h"

void crashout(char *bandicoot, ...)
{
	va_list arg;
	char msg[4096];
     
	if (bandicoot)
	{
		va_start(arg, bandicoot);
		vsprintf(msg, bandicoot, arg);
		va_end(arg);

		printf(msg);
		printf("\n\n");
	}
	
	exit(0);
}

void print_usage()
{
	printf("makeicns - Mac OSX .icns resource file generator for Allegro" 
			ALLEGRO_VERSION_STR "\n"
			"by Charles Wardlaw, " ALLEGRO_DATE_STR "\n\n"
			"Usage: makeicns infile.[pcx|tga|bmp] [outfile.icns]\n");
}

int main(int argc, char **argv)
{
	NSString *infile, *outfile;
	NSImage *image;
	IconFamily *iconfamily;
	char msg[4096];

	if (argc < 2)
	{
		print_usage();
		return 0;
	}

	// we have enough variables -- initialize allegro
	allegro_init();
	
	// set the name of the in file
	infile = [NSString stringWithCString:argv[1]];
	printf("using image from file: %s.\n", [infile cString]);

	if (argc > 2)
	{
		// we have a third argument -- use it as the output string
		outfile = [NSString stringWithCString:argv[2]];
	}
	else
	{
		// do some allegro magic on the filename
		replace_extension(msg, argv[1], "icns", 4095);
		outfile = [NSString stringWithCString:msg];
	}

	printf("using target file: %s.\n", [outfile cString]);

	// load the image ...
	image = [[NSImage alloc] initWithContentsOfFile:infile];
	if (!image)
		crashout("Error: couldn't load image from %s.", [infile cString]);

	// ... create the icon family ...
	iconfamily = [[IconFamily alloc] initWithThumbnailsOfImage:image];
	if (!iconfamily)
		crashout("Error: couldn't create icon family.", [infile cString]);
	
	// ... and save it to the new file
	if (![iconfamily writeToFile:outfile])
		crashout("Error: couldn't write to file %s.", [outfile cString]);
		
	// all's well, we hope
	
	printf("... done.\n\n");
	allegro_exit();
	return 0;	
}
END_OF_MAIN()